package mpi.eudico.client.annotator.interannotator;
/**
 * An interface defining constants for usage as step and preferences keys for 
 * inter-annotator agreement calculation wizard.
 *  
 * @author Han Sloetjes
 *
 */
public interface CompareConstants {

	/**
	 * An enumeration of available algorithms.
	 * 
	 * @author Han Sloetjes
	 */
	public static enum METHOD {
		/** classic */
		CLASSIC ("Classic"),
		/** modified kappa */
		MOD_KAPPA ("Modified Kappa"),
		/** staccato */
		STACCATO ("Staccato"),
		/** modified fleiss */
		MOD_FLEISS ("Modified Fleiss");
		
		/** value */
		public String value;
		
		private METHOD(String val) {
			this.value = val;
		}

		@Override
		public String toString() {
			return value;
		}
	}
	
	/** 
	 * An enumeration of File and Tier matching constants
	 * 
	 * @author Han Sloetjes
	 */
	public static enum MATCHING {
		/** manual */
		MANUAL ("Manual"),
		/** affix */
		AFFIX ("Affix based"),
		/** suffix */
		SUFFIX ("Suffix"),
		/** prefix */
		PREFIX ("Prefix"),
		/** same name */
		SAME_NAME ("Same name");
		
		/** value */
		public String value;
		
		private MATCHING(String val) {
			this.value = val;
		}

		@Override
		public String toString() {
			return value;
		}	
	}
	
	/** 
	 * An enumeration of File matching constants
	 * 
	 * @author Han Sloetjes
	 */
	public static enum FILE_MATCHING {
		/** Current document */
		CURRENT_DOC ("In current document"),
		/** in same file  */
		IN_SAME_FILE ("In same file"),
		/** across files */
		ACROSS_FILES ("Across files");
		
		/** value identifier */
		public String value;
		
		private FILE_MATCHING(String val) {
			this.value = val;
		}

		@Override
		public String toString() {
			return value;
		}	
	}

	// preferences keys
	/** method key constant */
	public static final String METHOD_KEY = "Compare.CompareMethod";
	/** file match key constant */
	public static final String FILE_MATCH_KEY = "Compare.FileMatching";
	/** tier match key constant */
	public static final String TIER_MATCH_KEY = "Compare.TierMatching";
	/** file separator key constant */
	public static final String FILE_SEPARATOR_KEY = "Compare.FileSeparator";
	/** tier separator key constant */
	public static final String TIER_SEPARATOR_KEY = "Compare.TierSeparator";
	/** tier source key constant */
	public static final String TIER_SOURCE_KEY = "Compare.TierSource";
	/** selected files key constant */
	public static final String SEL_FILES_KEY = "Compare.SelectedFiles";
	/** tier name1 key constant */
	public static final String TIER_NAME1_KEY = "Compare.TierName1";
	/** tier name2 key constant */
	public static final String TIER_NAME2_KEY = "Compare.TierName2";
	/** selected tier names constant */
	public static final String TIER_NAMES_KEY = "Compare.SelectedTierNames";
	/** all tier names constant */
	public static final String ALL_TIER_NAMES_KEY = "Compare.AllTierNames";
	/** overlap percentage constant */
	public static final String OVERLAP_PERCENTAGE = "Compare.OverlapPercentage";
	/** overlap average key constant */
	public static final String OVERLAP_AVERAGE = "Compare.OverlapAveragePercentage";
	/** output per file constant */
	public static final String OUTPUT_PER_FILE = "Compare.Output.PerFile";
	/** output per tier pair */
	public static final String OUTPUT_PER_TIER_PAIR = "Compare.Output.PerTierPair";
	/** output table values constant */
	public static final String OUTPUT_TABLES_VALUES = "Compare.Output.TablesOfValues";
	/** monte carlo simulation constant */
	public static final String MONTE_CARLO_SIM = "Compare.MonteCarloSimulations";
	/** number of nominations constant */
	public static final String NUM_NOMINATIONS = "Compare.NumberOfNominations";
	/** null hypothesis value constant */
	public static final String NULL_HYPOTHESIS = "Compare.NullHypothesis.Value";
	/** group compare key constant */
	public static final String GROUP_COMPARE_KEY = "Compare.GroupWise";// compare >=2 tiers
	/** export match tiers constant */
	public static final String EXPORT_MATCHING_TIERS_KEY = "Compare.ExportMatchingTiers";
	/** export tiers method key constant */
	public static final String EXPORT_TIERS_METHOD_KEY = "Compare.ExportTiersMethod";
	/** export tiers per tier set constant */
	public static final String EXPORT_TIERS_PERTIERSET = "Compare.ExportPerTierSet";
	/** export tiers per file constant */
	public static final String EXPORT_TIERS_PERFILE = "Compare.ExportPerFile";
	/** export folder key constant */
	public static final String EXPORT_FOLDER_KEY = "Compare.ExportFolder";
	
	// constant for unmatched or unlinked annotations
	/** unmatched constant */
	public static final String UNMATCHED = "Unmatched";
}
